% MTODOS NUMRICOS.
% ECUACIN DE CONVECCIN-DIFUSIN-REACCIN DEL TIPO LAX-WENDROFF DOS DIMENSIONES
%
% Aproxima la solucion sujeta a las condiciones de contorno
%                u(x,0) = 0, 0 < t < T = max t,
% y a las condiciones iniciales
%                u(x,y) = F(x,y), 0 <= x <= lx, 0 <= y <= ly:
%
% ENTRADA:   Longitud del alambre lx en x; nmero de puntos nx en x.
% ENTRADA:   Longitud del alambre ly en y; nmero de puntos ny en Y.
% ENTRADA:   Tiempo maximo T; constante de difusividad del medio K;
% ENTRADA:   Componentes de la velocidad del viento V[Vx Vy]; constante de reaccin R;
%
% SALIDA:  Aproximacion matriz_u(i,j) de f(x(i),y(i),t(j)) para cada
%          i = 1, ..., nx y j = 1, ..., ny.
 %------------------------------------------------------------------------%
 %                       BLOQUE DE DECLARACION DE VARIABLES               %
 %------------------------------------------------------------------------%
 syms('OK', 'lx', 'ly', 'T', 'K', 'nx', 'ny', 'nt', 'R', 'V_x', 'V_y', 'npuntos','z1','z2');
 syms('inc_x', 'inc_y', 'inc_t','i','j','t','punto','norte','sur','este','oeste','coeficiente','columna');
 syms('matriz_A','vector_b','vector_u','matriz_u', 'A', 'B', 'C', 'D', 'F','G');
 syms('cte_u','At', 'Bt', 'Ct', 'Dt', 'Ft','Gt','x','y','Funcion');
 syms('vector_x', 'TT', 'vector_y','OUP');
 TRUE = 1;
 FALSE = 0;
 %-------------------------------------------------------------------------%
 %                                PRESENTACION                             %
 %-------------------------------------------------------------------------%
 fprintf(1,'Este es el Metodo de planteamiento y la resolucin de una ecuacin de conveccin-difusin-reaccin\n');
 fprintf(1,'del tipo Lax-Wendroff a lo largo de un epsodio\n');
 %-------------------------------------------------------------------------%
 %                       BLOQUE DE ENTRADA DE DATOS                        %
 %-------------------------------------------------------------------------%
 fprintf(1,'Escriba la funcion F(X) (condicion inicial) en terminos de x e y.\n');
 fprintf(1,'Por ejemplo:  sin(pi*x*y)\n');
 % Lectura de la funcin.
 s = input(' ','s');
 Funcion = inline(s,'x','y');
 % Bucle para la lectura del valor en el eje X.
 fprintf(1,'El extremo izquierdo del eje X es 0.\n');
 OK = FALSE;
 while OK == FALSE 
     fprintf(1,'Escriba el extremo derecho del eje X (longitud de la malla en el eje x).\n');
     lx = input(' ');
     if lx <= 0 
        fprintf(1,'Debe ser un numero positivo.\n');
     else
        OK = TRUE;
     end;
 end;
 % Bucle para la lectura del valor en el eje Y.
 fprintf(1,'El extremo inferior del eje Y es 0.\n');
 OK = FALSE;
 while OK == FALSE 
     fprintf(1,'Escriba el extremo superior del eje Y (longitud de la malla en el eje y).\n');
     ly = input(' ');
     if ly <= 0 
        fprintf(1,'Debe ser un numero positivo.\n');
     else
        OK = TRUE;
     end;
 end;
 % Bucle para la lectura del valor maximo del tiempo.
 OK = FALSE;
 while OK == FALSE 
     fprintf(1,'Escriba el maximo valor T de la variable tiempo.\n');
     T = input(' ');
     if T <= 0 
        fprintf(1,'Debe ser un numero positivo.\n');
     else
        OK = TRUE;
     end;
 end;
 % Bucle para la lectura de los datos de entrada de la contaminacin.
 OK = FALSE;
 while OK == FALSE 
     fprintf(1,'Escriba el valor de la concentracin de la contaminacin (coordenadas del vector u).\n');
     cte_u = input(' ');
     if cte_u < 0 
        fprintf(1,'La concentracin debe ser mayor o igual a cero.\n');
     else
        OK = TRUE;
     end;
 end;
  % Bucle para la lectura de los datos de entrada del vector velocidad.
 OK = FALSE;
 while OK == FALSE 
     fprintf(1,'Escriba las coordenadas del vector velocidad, Vx e Vy en lneas separadas.\n');
     V_x = input(' ');
     V_y = input(' ');
     if V_x <= 0 | V_y <=0 
        fprintf(1,'Deben ser un numeros positivos.\n');
     else
        OK = TRUE;
     end;
 end;
 % Lectura de los datos de entrada de la constante de difusividad del medio y la constante de reaccin.
 fprintf(1,'Escriba la constante de difusividad del medio K > 0.\n');
 K = input(' ');
 % Lectura de los datos de entrada de la constante de reaccin.
 fprintf(1,'Escriba la constante de reaccin R > 0.\n');
 R = input(' ');
 % Lectura de los datos de entrada de los intervalos (en X, en Y y en el tiempo).
 OK = FALSE;
 while OK == FALSE 
     fprintf(1,'Escriba un entero nx = numero de particiones sobre el eje X\n');
     fprintf(1,'Escriba un entero ny = numero de particiones sobre el eje Y\n');
     fprintf(1,'y nt = numero de intervalos de tiempo - en lineas separadas.\n');
     fprintf(1,'En principio no hay restriccin para el valor. QUIZS HABRA QUE MIRARLO.\n');
     nx = input(' ');
     ny = input(' ');
     nt = input(' ');
     if nx < 2 |  ny < 2  |  T <= 0 
        fprintf(1,'Los numeros no estan dentro del rango adecuado.\n');
     else
        OK = TRUE;
     end;
 end;
 %-------------------------------------------------------------------------%
 %                   BLOQUE DE INICIALIZACIN DE VALORES                   %
 %-------------------------------------------------------------------------%
 if OK == TRUE
    
    % Nmero de puntos, matrices y vectores.     
    
    npuntos=nx*ny;
    matriz_A = zeros(npuntos);
    vector_b = zeros(1,npuntos);
    vector_u = zeros(1,npuntos);
    matriz_u = zeros(ny,nx);
    
    % Particiones (comenzando en 0).
    
    inc_x = lx/(nx-1);
    inc_y = ly/(ny-1);

    vector_x = [];
    for i=1:nx
        vector_x(i)=(i-1)*inc_x;
    end
    vector_y = [];
    for i=1:ny
        vector_y(i)=(i-1)*inc_y;
    end
    inc_t = T/(nt-1);
    
    % Eleccin de la forma de salida de los resultados.
    
    fprintf(1,'Elija la forma de salida de resultados:\n');
    fprintf(1,'1. Pantalla\n');
    fprintf(1,'2. Fichero de texto\n');
    fprintf(1,'Por favor escriba 1 o 2.\n');
    FLAG = input(' ');
    if FLAG == 2 
         fprintf(1,'Escriba el nombre del fichero de la forma - disco:\\nombre.ext\n');
         fprintf(1,'Por ejemplo:  A:\\OUTPUT.DTA\n');
         NAME = input(' ','s');
         OUP = fopen(NAME,'w+');
    else
         OUP = 1;
    end;
    
    % Impresin de la cabecera
    
    fprintf(OUP, '  j  i      X(i)      Y(i)      T(i)      vector_u(i) \n');
    
    % Obtencion de los puntos para la matriz_A a partir del mallado - Consultar frmulas.
    
    A=(-inc_t/(2*inc_x))*((K/inc_x)+(V_y/2));
    B=(-inc_t/2)*((K/(inc_y*inc_y))+(V_x/(2*inc_x)));
    C=(inc_t/2)*(((2*K)/(inc_y*inc_y))+((2*K)/(inc_x*inc_x))+R)+1;
    D=(inc_t/2)*(-K/(inc_y*inc_y)+(V_x/(2*inc_y)));
    F=(inc_t/2)*((K/(inc_x*inc_x))+(V_y/(2*inc_x)));
    
    % Almacenamiento de coeficientes para el vector_u en el punto interior del mallado.
    
    vector_u((round(ny / 2) - 1) * nx + round(nx / 2)) = cte_u;
    
    % Generacin de la matriz A (1 en la diagonal principal, y coeficientes -A,B,C,D,F- en puntos interiores).
    
    for j=1:ny
        for i=1:nx
            punto=(j-1)*nx+i;
            if (i == 1 |  j == 1  |  i == nx |  j == ny)
                % Condicin de contorno, elemento de la diagonal a 1
                matriz_A(punto,punto)=1; 
            else
                % Almacenamiento de coeficientes para la matriz en puntos interiores
                sur=punto-ny;
                oeste=punto-1;
                este=punto+1;
                norte=punto+ny;
                matriz_A(punto,sur)=A;
                matriz_A(punto,oeste)=B;
                matriz_A(punto,punto)=C;
                matriz_A(punto,este)=D;                 
                matriz_A(punto,norte)=F;                 
            end
        end
    end
    
    % Generacin de la matriz_u a partir del vector_u, para la funcin surf inicial.
    
    coeficiente=0;
    for z1=1:ny
        for z2=1:nx
            coeficiente=coeficiente+1;
            matriz_u(z1,z2)=vector_u(coeficiente);
            fprintf(OUP, '%3d %3d %2.8f %2.8f %2.8f ', z1, z2, vector_x(z2), vector_y(z1), 0);
            fprintf(OUP, ' %2.8f \n', vector_u(coeficiente));
        end
    end
    
    % Dibujo de la superficie en el momento inicial.
    
    figure(1);
    surf(vector_x,vector_y,matriz_u),colorbar;
    zlim([-1.1*cte_u 1.1*cte_u]);
    title('CONTAMINACIN INICIAL. Ecuacin de conveccin-difusin-reaccin.','FontAngle','italic','FontWeight','bold','Color',[1,0,0]);
    xlabel('Valores del eje X','FontSize',10,'FontAngle','italic','FontWeight','bold','Color',[0,0,1]);
    ylabel('Valores del eje Y','FontSize',10,'FontAngle','italic','FontWeight','bold','Color',[0,0,1]);
    zlabel('Valores de la funcin F(x,y)','FontSize',10,'FontAngle','italic','FontWeight','bold','Color',[0,0,1]');
    fprintf(1,'Pulse una tecla.\n');
    tecla = input(' ','s');
    
    % Obtencin de cada sistema de ecuaciones y resolucin del mismo en cada intervalo de tiempo.
    columna=0;
    for t=1:nt
         %paso en el tiempo
         columna=columna+1;
         TT(columna)=t*inc_t;
         for j=1:ny
          for i=1:nx
             punto=(j-1)*nx+i;
             if (i ~= 1 &  j ~= 1  &  i ~= nx &  j ~= ny)
                 % Obtencion de los puntos primados - Consultar frmulas.
                 At=(inc_t/2)*((K/inc_x)+(V_y/(2*inc_x)));
                 Bt=(inc_t/2)*((K/(inc_y*inc_y))+(V_x/(2*inc_x)));
                 Ct=1+(inc_t/2)*(((-2*K)/(inc_y*inc_y))+((-2*K)/(inc_x*inc_x))-R);
                 Dt=(inc_t/2)*(K/(inc_y*inc_y)+(V_x/(2*inc_y)));
                 Ft=(inc_t/2)*((K/(inc_x*inc_x))+(V_y/(2*inc_x)));
                 sur=punto-ny;
                 oeste=punto-1;
                 este=punto+1;
                 norte=punto+ny;
                 G=At*vector_u(sur)+Bt*vector_u(oeste)+Ct*vector_u(punto)+Dt*vector_u(este)+Ft*vector_u(norte);
                 x=(i-1)*inc_x;
                 y=(j-1)*inc_y;
                 Gt=(inc_t/2)*Funcion(x,y);
                 vector_b(punto)=G+Gt;
             end
          end
         end
         
         % Resolucin del sistema de ecuaciones utilizando el comando bicgstab.
         
         vector_u = bicgstab(matriz_A,transpose(vector_b));
                  
         % Generacin de la matriz_u a partir del vector_u, para la funcin surf.
         
         coeficiente=0;
         for z1=1:ny
             for z2=1:nx
                 coeficiente=coeficiente+1;
                 matriz_u(z1,z2)=vector_u(coeficiente);
                 fprintf(OUP, '%3d %3d %2.8f %2.8f %8s ', z1, z2, vector_x(z2), vector_y(z1), char(TT(columna)));
                 fprintf(OUP, ' %2.8f \n', vector_u(coeficiente));
             end
         end
         
         % Dibujo de la superficie en cada intervalo de tiempo.
         
         figure(1);
         surf(vector_x,vector_y,matriz_u),colorbar;
         zlim([-1.1*cte_u 1.1*cte_u]);
         x=TT(columna);
         title(['Contaminacin. Ecuacin de conveccin-difusin-reaccin en el instante de tiempo: ',num2str(double(x))],'FontAngle','italic','FontWeight','bold','Color',[1,0,0]);
         xlabel('Valores del eje X','FontSize',10,'FontAngle','italic','FontWeight','bold','Color',[0,0,1]);
         ylabel('Valores del eje Y','FontSize',10,'FontAngle','italic','FontWeight','bold','Color',[0,0,1]);
         zlabel('Valores de la funcin F(x,y)','FontSize',10,'FontAngle','italic','FontWeight','bold','Color',[0,0,1]');
         hold on;
         fprintf(1,'Pulse una tecla.\n');
         tecla = input(' ','s');
         hold off;
    end;
 end;
 
 % Si se eligi fichero como forma de salida, mensaje de finalizacin.
 
 if OUP ~= 1 
     fclose(OUP);
     fprintf(1,'Fichero de salida creado satisfactoriamente \n',NAME);
 end;
